{{
  Clock Functions

  Keep the master time and date values updated, provide methods
  to read and set them, and 
}}

obj

  vid       : "vga_40x18_rom_text"
  tfmt      : "tdfmt"
  bsf       : "bufstrfmt"

DAT

{
  Master system time and date in two longs
}
SysTime    long  0 'sec (0-59) + min (0-59) << 8 + hour (0-23) << 16  
SysDate    long  0 'day (1-7) + date (1-31) << 8 + month (1-12) << 16 + year (0-99) << 24 

C0         long  0
S0         byte  $FF

PUB Start

  systime := 0
  sysdate := $010101  'day of week, date, and month cannot be zero

  c0 := cnt

Pub Service | s
{
  This must be called regularly when the system is waiting
  for input to keep the clock time and display updated
}
  Advance
  s := tfmt.second(systime)
  if s <> s0
    'redraw when second changes
    Display
    s0 := s

PUB ReadTime

  return systime

PUB ReadDate

  return sysdate

PUB SetTime(newval)

  systime := newval

PUB SetDate(newval)

  sysdate := newval

PUB Display | ptr, col, row, color
{
  Show the time and date at the bottom of the screen
}
  ptr := tfmt.tfmt(systime)
  ptr := bsf.chcat(ptr, " ")
  ptr := bsf.strcat(ptr, tfmt.dfmt(sysdate))
  'save position and color
  col := vid.getcol
  row := vid.getrow
  color := vid.getcolor
  vid.setpos(0, vid#rows-1)
  'and center the whole thing on screen
  'set and print in primary color for this line
  vid.str(string($C,%%2020,$C,%100))
  vid.str(bsf.cstr(ptr, vid#cols))
  'restore position and color
  vid.setpos(col, row)
  vid.setcolor(color)
   

PUB advance | time, date, h, m, s, dw, d, mo, ml, y
{
  Engine which advances the time and date each second.  This should
  be called once a second.  

  In a real system this would be replaced by a routine which reads
  an actual hardware time and date chip.
}

  'Note the circular nature of 2's complement math means that this
  'comparison will work even if cnt has rolled over.  This will
  'even keep the correct time if the function isn't called for
  'several seconds, as long as cnt does not advance enough to
  'flip the sign of the result (about 40 seconds at 80 MHz)
  '
  repeat while (cnt - c0) > clkfreq
    c0 += clkfreq
    '
    'this happens every second
    '
    s := tfmt.second(systime)
    s++
    if s > 59
      s := 0
      m := tfmt.minute(systime)
      m++
      if m > 59
        m := 0
        h := tfmt.hour(systime)
        h++
        if h > 23
          h := 0
          dw := tfmt.day(sysdate)
          dw++
          if dw > 7
            dw := 1
          sysdate := tfmt.setday(sysdate, dw)
          d := tfmt.date(sysdate)
          mo := tfmt.month(sysdate)
          y := tfmt.year(sysdate)
          d++
          ml := lookup(mo: 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31)
          if ml == 28 and (y & %11) == 0
            ml++
          if d > ml
            d := 1
            mo++
            if mo > 12
              mo := 1
              y++
              if y > 99
                y := 0
              sysdate := tfmt.setyear(sysdate, y)  
            sysdate := tfmt.setmonth(sysdate, mo)
          sysdate := tfmt.setdate(sysdate, d)                    
        systime := tfmt.sethour(systime, h)
      systime := tfmt.setminute(systime, m)
    systime := tfmt.setsecond(systime, s)

pub Bump(field, dir) | td, vmin, vmax, v
{
  Set the time or date by bumping the given field up or down,
  rolling at the appropriate point to keep it valid.
}
  case field
    0..2:
      td := systime
    other:
      td := sysdate

  vmin := 0    
  case field
    0:
      v := tfmt.second(td)
      vmax := 59
    1:
      v := tfmt.minute(td)
      vmax := 59
    2:
      v := tfmt.hour(td)
      vmax := 23
    3:
      v := tfmt.day(td)
      vmin := 1
      vmax := 7
    4:
      v := tfmt.date(td)
      vmin := 1
      vmax := lookup(tfmt.month(td): 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31)
      if vmax == 28 and (tfmt.year(td) & %11) == 0
        vmax++
    5:
      v := tfmt.month(td)
      vmin := 1
      vmax := 12
    6:
      v := tfmt.year(td)
      vmin := 0
      vmax := 99

  v += dir
  if v < vmin
    v := vmax
  elseif v > vmax
    v := vmin

  case field
    0:
      td := tfmt.setsecond(td, v)
    1:
      td := tfmt.setminute(td, v)
    2:
      td := tfmt.sethour(td, v)
    3:
      td := tfmt.setday(td, v)
    4:
      td := tfmt.setdate(td, v)
    5:
      td := tfmt.setmonth(td, v)
    6:
      td := tfmt.setyear(td, v)

  case field
    0..2:
      systime := td
    other:
      sysdate := td
    